<?php
/*
a(tom)Blog is a atom powered weblog system.
Copyright (C) 2004  Yoan BLANC

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
/**
 * An universal tool to do transformation (support Sablotron, Xsltproc and DomXml)
 *
 * utilisation : 
 *	
 *	<pre>$xslt = new Xslt(["/data/greut/2004/020/29_233000.xml"[, "/xsl/toEntry.xsl"[, array("a"=>3,"b"=>2)[, "sablotron"]]]]);
 *
 *	$xslt->setEngine("sablotron");
 *	$xslt->setXml("/data/greut/2004/020/29_233000.xml");
 *	$xslt->setXsl("/xsl/toEntry.xsl");
 *	$xslt->setParam(array("a"=>3,"b"=>2));
 *
 *	$result = $xslt->transform();</pre>
 *
 *	informations (html format) :
 *
 *	<pre>echo $xslt->toString();</pre>
 *
 * @author	Yoan Blanc <greut [at] altern.org>
 * @version 0.1 040625
 */
class Xslt
{
	/// XML file
	var $xml;
	/// Stylesheet
	var $xsl;
	/// Parameters
	var $param;
	
	/// chosen engine
	var $engine;
	/// supported engines
	var $xsltEngines;
	
	/// transformation result
	var $result;
	
	/**
	 * Init the system
	 *
	 * @param $xml XML file
	 * @param $xsl Stylesheet
	 * @param $param transformation parameters
	 * @param $engine XSLT engine used
	 */
	function Xslt($xml="",$xsl="",$param=NULL,$engine="sablotron")
	{
		$this->xsltEngines = array("sablotron"=>1,"domxml"=>1,"xsltproc"=>1);
		
		$this->setXml($xml);
		$this->setXsl($xsl);
		$this->setParam($param);
		$this->setEngine($engine);
	}
	
	/**
	 * Change the used engine
	 *
	 * @param $engine engine name
	 */
	function setEngine($engine)
	{
		$engine = strtolower($engine);
		if(isset($this->xsltEngines[$engine]) && $this->xsltEngines[$engine]==1)
		{
			$this->engine = $engine;
		}
		else
		{
			return false;
		}
	}
	
	/**
	 * Return the current engine
	 *
	 * @return engine used
	 */
	function getEngine()
	{
		return $this->engine;
	}
	
	/**
	 * Do the transformation
	 *
	 * @see result
	 */
	function transform()
	{
		switch ($this->engine)
		{
			case "sablotron":
				$engine = new Sablotron($this->xml, $this->xsl, $this->param);
				break;
			case "domxml":
				$engine = new Domxml($this->xml, $this->xsl, $this->param);
				break;
			case "xsltproc":
				$engine = new Xsltproc($this->xml, $this->xsl, $this->param);
				break;
		}
		
		if(isset($engine))
		{
			$engine->transform();
			$this->result = $engine->result();
		}
		else
		{
			$this->result = '<error><msg>xlst.class.php:Xslt::transform()</msg></error>';
		}
	}
	
	/**
	 * Define the XML file
	 *
	 * @param $xml XML file
	 */
	function setXml($xml="")
	{
		if(file_exists($xml))
		{
			$this->xml = $xml;
			return true;
		}
		else
		{
			return false;
		}
	}
	
	/**
	 * Define the XSL file
	 *
	 * @param $xsl stylesheet
	 */
	function setXsl($xsl="")
	{
		if(file_exists($xsl))
		{
			$this->xsl = $xsl;
			return true;
		}
		else
		{
			return false;
		}
	}
	
	/**
	 * Define the transformation parameters
	 *
	 * @param $param parameters
	 */
	function setParam($param=NULL)
	{
		if(is_array($param))
		{
			$this->param = $param;
		}
		else
		{
			$this->param = array();
		}
	}
	
	/**
	 * Informations about the current state of this object
	 *
	 * @return html formatted informations
	 */
	function toString()
	{
		$ret = "
			<dl>
				<dt>xml</dt>
				<dd>".$this->xml."</dd>
				<dt>xsl</dt>
				<dd>".$this->xsl."</dd>";
		if(count($this->param) > 0)
		{
			$ret .= "
					<dt>param</dt>
					<dd><dl>";
			foreach($this->param as $key => $value)
			{
				$ret .= "
						<dt>".$key."</dt>
						<dd>".$value."</dd>";
			}
			$ret .= "</dl></dd>";
		}
		$ret .= "
				<dt>engine</dt>
				<dd><dl>";
		foreach($this->xsltEngines as $key => $value)
		{
			$ret .= "
					<dt>".$key."</dt>
					<dd>";
			if($key == $this->engine)
			{
				$ret .= "selected";
			}
			else if($value == 0)
			{
				$ret .= "unactive";
			}
			else
			{
				$ret .= "active";
			}
				$ret .= "</dd>";
		}
		$ret .= "</dl></dd>
			</dl>
		";
		return $ret;
	}
	
	/**
	 * Return the result
	 *
	 * @return transformation result
	 */
	function result()
	{
		return $this->result;
	}
}

/** [abstract class]
 * apply transformation for specific engine
 *
 * @author	Yoan Blanc <greut [at] altern.org>
 * @version 0.1 040625
 */
class xslt_base
{
	/// XML file
	var $xml;
	/// stylesheet
	var $xsl;
	/// parameters
	var $param;
	/// transformation's result
	var $result;
	
	/**
	 * Construct the transformation tool
	 * 
	 * @param $xml XML file
	 * @param $xsl stylesheet
	 * @param $param parameters
	 */
	function xslt_base($xml, $xsl, $param=array())
	{
		$this->xml = $xml;
		$this->xsl = $xsl;
		$this->param = $param;
	}
	
	/** [abstract method]
	 * Do the transformation
	 */
	function transform()
	{
		return "<error><msg>this is an abstract class</msg></error>";
	}
	
	/**
	 * Return the transformation result
	 *
	 * @return result
	 */
	function result()
	{
		return $this->result;
	}
}

/**
 * XSLT transformation with Sablotron
 *
 * @author	Yoan Blanc <greut [at] altern.org>
 * @version 0.1 040625
 */
class Sablotron extends xslt_base
{
	/// XSLT processor resource
	var $xh;
	
	/**
	 * Construct the transformation tool
	 * 
	 * @param $xml XML file
	 * @param $xsl stylesheet
	 * @param $param parameters
	 */
	function Sablotron($xml, $xsl, $param=array())
	{
		parent::xslt_base($xml, $xsl, $param);
		$this->xh = xslt_create();
	}
	
	/**
	 * Apply the transformation
	 */
	function transform()
	{
		if(!($this->result = xslt_process($this->xh, $this->xml, $this->xsl,NULL,array(),$this->param)))
		{
			$this->result = '<error><msg>'.xslt_error($this->xh).'</msg><num>'.xslt_errno($this->xh).')</num></error>';
		}
		xslt_free($this->xh);
	}
}
/**
 * XSLT transformation with DomXML
 *
 * @author	Yoan Blanc <greut [at] altern.org>
 * @version 0.1 040625
 */
class Domxml extends xslt_base
{
	/**
	 * Construct the transformation tool
	 * 
	 * @param $xml XML file
	 * @param $xsl stylesheet
	 * @param $param parameters
	 */
	function Domxml($xml, $xsl, $param=array())
	{
		parent::xslt_base($xml, $xsl, $param);
	}
	
	/**
	 * Apply the transformation
	 */
	function transform()
	{
		$dom_xslt = domxml_xslt_stylesheet_file($this->xsl);
		$dom_xml = domxml_open_file($this->xml);
		$dom_data = $dom_xslt->process($dom_xml,$this->param);
		//$this->result = $dom_data->dump_mem();
		$this->result = $dom_xslt->result_dump_mem($dom_data);
	}
}

/**
 * XSLT transformation with Xsltproc
 *
 * @author	Yoan Blanc <greut [at] altern.org>
 * @version 0.1 040625
 */
class Xsltproc extends xslt_base
{
	/**
	 * Construct the transformation tool
	 * 
	 * @param $xml XML file
	 * @param $xsl stylesheet
	 * @param $param parameters
	 */
	function Xsltproc($xml, $xsl, $param=array())
	{
		parent::xslt_base($xml, $xsl, $param);
		$this->proc = '/usr/bin/xsltproc';
	}
	
	/**
	 * Apply the transformation
	 */
	function transform()
	{
		exec($this->proc.$this->__paramFormat()." ".$this->xsl." ".$this->xml." 2>&1",$result);
		$this->result = implode("\n",$result);
	}
	
	/** [private method]
	 * Formatting the parameters for execution
	 *
	 * @return formatted parameters
	 */
	function __paramFormat()
	{
		$string = "";
		if(count($this->param)>0)
		{
			foreach($this->param as $key => $value)
			{
				$string .= ' --stringparam '.$key.' "'.$value.'"';
			}
		}
		return $string;
	}
}
?>